export declare type Decoder<T> = (input: unknown) => T;
/**
 * Try all the given decoders and return the result of the first one not raising an error.
 * If none succeed, throws a {@link OneOfDecoderError}
 *
 * For example, this is how nullable is defined:
 * ```ts
 * const nullable = <T>(decoder: Decoder<T>) => decod.oneOf(decoder, decod.null_);
 * ```
 *
 * @category Decoder combinators
 */
export declare function oneOf<D1>(d1: Decoder<D1>): Decoder<D1>;
export declare function oneOf<D1, D2>(d1: Decoder<D1>, d2: Decoder<D2>): Decoder<D1 | D2>;
export declare function oneOf<D1, D2, D3>(d1: Decoder<D1>, d2: Decoder<D2>, d3: Decoder<D3>): Decoder<D1 | D2 | D3>;
export declare function oneOf<D1, D2, D3, D4>(d1: Decoder<D1>, d2: Decoder<D2>, d3: Decoder<D3>, d4: Decoder<D4>): Decoder<D1 | D2 | D3 | D4>;
export declare function oneOf<D1, D2, D3, D4, D5>(d1: Decoder<D1>, d2: Decoder<D2>, d3: Decoder<D3>, d4: Decoder<D4>, d5: Decoder<D5>): Decoder<D1 | D2 | D3 | D4 | D5>;
export declare function oneOf<D1, D2, D3, D4, D5, D6>(d1: Decoder<D1>, d2: Decoder<D2>, d3: Decoder<D3>, d4: Decoder<D4>, d5: Decoder<D5>, d6: Decoder<D6>): Decoder<D1 | D2 | D3 | D4 | D5 | D6>;
export declare function oneOf<D1, D2, D3, D4, D5, D6, D7>(d1: Decoder<D1>, d2: Decoder<D2>, d3: Decoder<D3>, d4: Decoder<D4>, d5: Decoder<D5>, d6: Decoder<D6>, d7: Decoder<D7>): Decoder<D1 | D2 | D3 | D4 | D5 | D6 | D7>;
export declare function oneOf<D1, D2, D3, D4, D5, D6, D7, D8>(d1: Decoder<D1>, d2: Decoder<D2>, d3: Decoder<D3>, d4: Decoder<D4>, d5: Decoder<D5>, d6: Decoder<D6>, d7: Decoder<D7>, d8: Decoder<D8>): Decoder<D1 | D2 | D3 | D4 | D5 | D6 | D7 | D8>;
export declare function oneOf<D1, D2, D3, D4, D5, D6, D7, D8, D9>(d1: Decoder<D1>, d2: Decoder<D2>, d3: Decoder<D3>, d4: Decoder<D4>, d5: Decoder<D5>, d6: Decoder<D6>, d7: Decoder<D7>, d8: Decoder<D8>, d9: Decoder<D9>): Decoder<D1 | D2 | D3 | D4 | D5 | D6 | D7 | D8 | D9>;
/**
 * Decode an `unknown` value as a `string`.
 * Otherwise, throws a {@link ScalarDecoderError}.
 *
 * @category Primitive decoders
 */
export declare const string: Decoder<string>;
/**
 * Decode an `unknown` value if it matches the expected value.
 * Otherwise, throws a {@link ScalarDecoderError}.
 *
 * It is mostly intended to be used in conjunction with {@link oneOf} for decoding groups of specific values.
 *
 * ```ts
 * const droidDecoder = decod.oneOf(
 *   decod.is('r2d2'),
 *   decod.is('c3po')
 * );
 * ```
 *
 * @category Primitive decoders
 */
export declare const is: <T extends string | number | boolean | null | undefined>(expectedValue: T) => Decoder<T>;
/**
 * Decode an `unknown` value as a `number`.
 * Otherwise, throws a {@link ScalarDecoderError}.
 *
 * @category Primitive decoders
 */
export declare const number: Decoder<number>;
/**
 * Decode an `unknown` value as a `null` value.
 * Otherwise, throws a {@link ScalarDecoderError}.
 *
 * @category Primitive decoders
 */
export declare const null_: Decoder<null>;
/**
 * Decode an `unknown` value as `undefined`.
 * Otherwise, throws a {@link ScalarDecoderError}.
 *
 * @category Primitive decoders
 */
export declare const undefined_: Decoder<undefined>;
/**
 * Transform a `Decoder<T>` into a `Decoder<T | null | undefined>`.
 *
 * @category Decoder combinators
 */
export declare const optional: <T>(decoder: Decoder<T>) => Decoder<T | null | undefined>;
/**
 * Transform a `Decoder<T>` into a `Decoder<T | null>`.
 *
 * @category Decoder combinators
 */
export declare const nullable: <T>(decoder: Decoder<T>) => Decoder<T | null>;
/**
 * Decode an `unknown` value as a `boolean`.
 * Otherwise, throws a {@link ScalarDecoderError}.
 *
 * @category Primitive decoders
 */
export declare const boolean: Decoder<boolean>;
/**
 * Decode an `unknown` value as a `Date`.
 * Otherwise, throws a {@link ScalarDecoderError}.
 *
 * @category Primitive decoders
 */
export declare const date: Decoder<Date>;
/**
 * Apply the provided decoder at the specified path in a complex object.
 * Path can be either:
 * - a `string` (when querying a top level field)
 * - a `number` (when querying an array index)
 * - an `Array<string | number>` (to query an arbitrarily nested field)
 *
 * @category Decoder combinators
 */
export declare const at: <T>(path: string | number | (string | number)[], decoder: Decoder<T>) => Decoder<T>;
/**
 * Transform a `Decoder<T>` into a `Decoder<Array<T>>`.
 *
 * @category Decoder combinators
 */
export declare const array: <T>(decoder: Decoder<T>) => Decoder<T[]>;
/**
 * Try a decoder, and fallback to a specified value (if provided) if the decoder fails.
 * If no default value is specified, fallback to `undefined`.
 *
 * @category Decoder combinators
 */
export declare function try_<T>(decoder: Decoder<T>): Decoder<T | undefined>;
export declare function try_<T>(decoder: Decoder<T>, defaultValue: T): Decoder<T>;
/**
 * Synonym for {@link try_}
 *
 * @category Decoder combinators
 */
export declare const attempt: typeof try_;
/**
 * Create a decoder for arbitrary key/value pairs from a key decoder and a value decoder.
 * Effectively transforms a `Decoder<K>` and a `Decoder<V>` into a `Decoder<Array<{ key: K, value: V }>>`.
 *
 * @category Decoder combinators
 */
export declare const assoc: <K, V, T extends {
    [key: string]: unknown;
}>(keyDecoder: Decoder<K>, valueDecoder: Decoder<V>) => (input: T) => {
    key: K;
    value: V;
}[];
/**
 * Create a decoder for some complex structure from an object where each value is a `Decoder` and the keys
 * correspond to the desired output structure.
 *
 * ```ts
 * const personDecoder = decod.props({
 *   first_name: decod.at('first_name', decod.string),
 *   last_name: decod.at('last_name', decod.string),
 *   age: decod.at('age', decod.number),
 * });
 * ```
 *
 * @category Decoder combinators
 */
export declare const props: <T extends {
    [key: string]: Decoder<any>;
}>(obj: T) => (input: unknown) => { [P in keyof T]: ReturnType<T[P]>; };
//# sourceMappingURL=decoders.d.ts.map