var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
import { ArrayDecoderError, AtDecoderError, OneOfDecoderError, PropsDecoderError, ScalarDecoderError, StrictDecoderError, } from './errors';
export function oneOf() {
    var decoders = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        decoders[_i] = arguments[_i];
    }
    return function (input) {
        var errors = [];
        for (var _i = 0, decoders_1 = decoders; _i < decoders_1.length; _i++) {
            var decoder = decoders_1[_i];
            try {
                return decoder(input);
            }
            catch (e) {
                errors.push(e);
            }
        }
        throw new OneOfDecoderError(errors);
    };
}
/**
 * Decode an `unknown` value as a `string`.
 * Otherwise, throws a {@link ScalarDecoderError}.
 *
 * @category Primitive decoders
 */
export var string = function (input) {
    if (typeof input === 'string') {
        return input;
    }
    throw new ScalarDecoderError('string', input);
};
/**
 * Decode an `unknown` value if it matches the expected value.
 * Otherwise, throws a {@link ScalarDecoderError}.
 *
 * It is mostly intended to be used in conjunction with {@link oneOf} for decoding groups of specific values.
 *
 * ```ts
 * const droidDecoder = decod.oneOf(
 *   decod.is('r2d2'),
 *   decod.is('c3po')
 * );
 * ```
 *
 * @category Primitive decoders
 */
export var is = function (expectedValue) { return function (input) {
    if (input === expectedValue) {
        return expectedValue;
    }
    throw new StrictDecoderError(expectedValue, input);
}; };
/**
 * Decode an `unknown` value as a `number`.
 * Otherwise, throws a {@link ScalarDecoderError}.
 *
 * @category Primitive decoders
 */
export var number = function (input) {
    if (typeof input === 'number') {
        return input;
    }
    throw new ScalarDecoderError('number', input);
};
/**
 * Decode an `unknown` value as a `null` value.
 * Otherwise, throws a {@link ScalarDecoderError}.
 *
 * @category Primitive decoders
 */
export var null_ = function (input) {
    if (input === null) {
        return input;
    }
    throw new ScalarDecoderError('null', input);
};
/**
 * Decode an `unknown` value as `undefined`.
 * Otherwise, throws a {@link ScalarDecoderError}.
 *
 * @category Primitive decoders
 */
export var undefined_ = function (input) {
    if (typeof input === 'undefined') {
        return input;
    }
    throw new ScalarDecoderError('undefined', input);
};
/**
 * Transform a `Decoder<T>` into a `Decoder<T | null | undefined>`.
 *
 * @category Decoder combinators
 */
export var optional = function (decoder) {
    return oneOf(decoder, null_, undefined_);
};
/**
 * Transform a `Decoder<T>` into a `Decoder<T | null>`.
 *
 * @category Decoder combinators
 */
export var nullable = function (decoder) { return oneOf(decoder, null_); };
/**
 * Decode an `unknown` value as a `boolean`.
 * Otherwise, throws a {@link ScalarDecoderError}.
 *
 * @category Primitive decoders
 */
export var boolean = function (input) {
    if (typeof input === 'boolean') {
        return input;
    }
    throw new ScalarDecoderError('boolean', input);
};
/**
 * Decode an `unknown` value as a `Date`.
 * Otherwise, throws a {@link ScalarDecoderError}.
 *
 * @category Primitive decoders
 */
export var date = function (input) {
    if (input instanceof Date) {
        return input;
    }
    throw new ScalarDecoderError('Date', input);
};
/**
 * Apply the provided decoder at the specified path in a complex object.
 * Path can be either:
 * - a `string` (when querying a top level field)
 * - a `number` (when querying an array index)
 * - an `Array<string | number>` (to query an arbitrarily nested field)
 *
 * @category Decoder combinators
 */
export var at = function (path, decoder) { return function (input) {
    try {
        if (typeof path === 'string' || typeof path === 'number') {
            return decoder(input[path]);
        }
        if (path.length) {
            return at(path.slice(1), decoder)(input[path[0]]);
        }
    }
    catch (e) {
        if (typeof path === 'string' || typeof path === 'number') {
            throw new AtDecoderError(path, e);
        }
        throw new AtDecoderError(path[0], e);
    }
    return decoder(input);
}; };
/**
 * Transform a `Decoder<T>` into a `Decoder<Array<T>>`.
 *
 * @category Decoder combinators
 */
export var array = function (decoder) { return function (input) {
    if (Array.isArray(input)) {
        return input.map(function (value, index) {
            try {
                return decoder(value);
            }
            catch (e) {
                throw new ArrayDecoderError(index, e);
            }
        });
    }
    throw new ScalarDecoderError('array', input);
}; };
export function try_(decoder, defaultValue) {
    return oneOf(decoder, function (_) { return defaultValue; });
}
/**
 * Synonym for {@link try_}
 *
 * @category Decoder combinators
 */
export var attempt = try_;
/**
 * Create a decoder for arbitrary key/value pairs from a key decoder and a value decoder.
 * Effectively transforms a `Decoder<K>` and a `Decoder<V>` into a `Decoder<Array<{ key: K, value: V }>>`.
 *
 * @category Decoder combinators
 */
export var assoc = function (keyDecoder, valueDecoder) { return function (input) {
    return Object.keys(input).map(function (key) { return ({
        key: keyDecoder(key),
        value: valueDecoder(input[key]),
    }); });
}; };
/**
 * Create a decoder for some complex structure from an object where each value is a `Decoder` and the keys
 * correspond to the desired output structure.
 *
 * ```ts
 * const personDecoder = decod.props({
 *   first_name: decod.at('first_name', decod.string),
 *   last_name: decod.at('last_name', decod.string),
 *   age: decod.at('age', decod.number),
 * });
 * ```
 *
 * @category Decoder combinators
 */
export var props = function (obj) { return function (input) {
    return Object.keys(obj).reduce(function (acc, key) {
        var _a;
        try {
            var decoded = obj[key](input);
            return __assign(__assign({}, acc), (_a = {}, _a[key] = decoded, _a));
        }
        catch (e) {
            throw new PropsDecoderError(key, e);
        }
    }, {});
}; };
//# sourceMappingURL=decoders.js.map