import * as decod from './index';
describe('Decod', function () {
    it('Should decode strings', function () {
        expect(function () { return decod.string('Hello'); }).not.toThrowError();
        expect(function () { return decod.string(12); }).toThrowError("expected type: \"string\" but got type: \"number\" for value: \"12\"");
        expect(decod.string('Hello')).toEqual('Hello');
    });
    it('Should decode strict values', function () {
        expect(function () { return decod.is('Hello')('Hello'); }).not.toThrowError();
        expect(function () { return decod.is(12)('12'); }).toThrowError("expected value: \"12\" of type: \"number\" but got value: \"12\" of type: \"string\"");
        expect(decod.is('Hello')('Hello')).toEqual('Hello');
    });
    it('Should decode numbers', function () {
        expect(function () { return decod.number(12); }).not.toThrowError();
        expect(function () { return decod.number('12'); }).toThrowError("expected type: \"number\" but got type: \"string\" for value: \"12\"");
        expect(decod.number(12)).toEqual(12);
    });
    it('Should decode booleans', function () {
        expect(function () { return decod.boolean(false); }).not.toThrowError();
        expect(function () { return decod.boolean('32'); }).toThrowError("expected type: \"boolean\" but got type: \"string\" for value: \"32\"");
        expect(decod.boolean(false)).toEqual(false);
    });
    it('Should decode dates', function () {
        expect(function () { return decod.date(new Date()); }).not.toThrowError();
        expect(function () { return decod.date(32); }).toThrowError("expected type: \"Date\" but got type: \"number\" for value: \"32\"");
        var date = new Date();
        expect(decod.date(date)).toEqual(date);
    });
    it('Should decode nulls', function () {
        expect(function () { return decod.null_(null); }).not.toThrowError();
        expect(function () { return decod.null_('32'); }).toThrowError("expected type: \"null\" but got type: \"string\" for value: \"32\"");
        expect(decod.null_(null)).toEqual(null);
    });
    it('Should follow the path', function () {
        expect(decod.at(2, decod.string)(['foo', 'bar', 'baz'])).toEqual('baz');
        expect(decod.at('foo', decod.string)({ foo: 'bar', quux: 'baz' })).toEqual('bar');
        expect(decod.at(['foo', 1, 'bar'], decod.number)({
            foo: [{ bar: 0 }, { bar: 42 }],
        })).toEqual(42);
    });
    it('Should use the props helper', function () {
        expect(function () { return decod.props({})('hello'); }).not.toThrowError();
        expect(function () { return decod.props({ val: decod.number })('hello'); }).toThrowError("expected type: \"number\" but got type: \"string\" for value: \"hello\"");
        expect(decod.props({
            one: decod.at('one', decod.string),
            two: decod.at(['two', 'nested'], decod.number),
            three: decod.at(['three', 1], decod.string),
        })({ one: 'one', two: { nested: 2 }, three: ['n/a', 'three'] })).toEqual({ one: 'one', two: 2, three: 'three' });
    });
    it('Should decode nullables', function () {
        expect(function () { return decod.nullable(decod.string)(null); }).not.toThrowError();
        expect(function () { return decod.nullable(decod.string)('null'); }).not.toThrowError();
        expect(function () { return decod.nullable(decod.string)(32); }).toThrowError("all decoders failed\n\tScalarDecoderError: expected type: \"string\" but got type: \"number\" for value: \"32\"\n\tScalarDecoderError: expected type: \"null\" but got type: \"number\" for value: \"32\"");
        expect(decod.nullable(decod.string)('null')).toBe('null');
        expect(decod.nullable(decod.string)(null)).toBe(null);
    });
    it('Should decode optionals', function () {
        expect(function () { return decod.optional(decod.string)(null); }).not.toThrowError();
        expect(function () { return decod.optional(decod.string)('null'); }).not.toThrowError();
        expect(function () { return decod.optional(decod.string)(32); }).toThrowError("all decoders failed\n\tScalarDecoderError: expected type: \"string\" but got type: \"number\" for value: \"32\"\n\tScalarDecoderError: expected type: \"null\" but got type: \"number\" for value: \"32\"\n\tScalarDecoderError: expected type: \"undefined\" but got type: \"number\" for value: \"32\"");
        expect(decod.optional(decod.string)('null')).toBe('null');
        expect(decod.optional(decod.string)(null)).toBe(null);
        expect(decod.optional(decod.string)(undefined)).toBe(undefined);
    });
    it('Should decode undefined', function () {
        expect(function () { return decod.undefined_(undefined); }).not.toThrowError();
        expect(function () { return decod.undefined_('32'); }).toThrowError("expected type: \"undefined\" but got type: \"string\" for value: \"32\"");
        expect(decod.undefined_(undefined)).toEqual(undefined);
    });
    it('Should try to decode', function () {
        expect(function () { return decod.try_(decod.string)('32'); }).not.toThrowError();
        expect(function () { return decod.try_(decod.string)(32); }).not.toThrowError();
        expect(decod.try_(decod.string)('32')).toEqual('32');
    });
    it('Should decode assoc', function () {
        var valueDecoder = decod.oneOf(decod.string, decod.number);
        var decoder = decod.assoc(decod.string, valueDecoder);
        expect(function () { return decoder({ a: 'b' }); }).not.toThrowError();
        expect(decoder({ a: 'one', b: 1 })).toEqual([
            {
                key: 'a',
                value: 'one',
            },
            {
                key: 'b',
                value: 1,
            },
        ]);
    });
    it('Should decode objects', function () {
        expect(function () {
            return decod.at(['a', 'b', 1, 2], decod.string)({ a: { b: { 1: { 2: '' } } } });
        }).not.toThrowError();
        expect(function () { return decod.at(['a'], decod.string)(''); }).toThrowError("at path: \"a\"\n\tScalarDecoderError: expected type: \"string\" but got type: \"undefined\" for value: \"undefined\"");
        expect(decod.at(['a'], decod.string)({ a: 'Hello' })).toEqual('Hello');
    });
    it('Should decode arrays', function () {
        expect(function () {
            return decod.array(decod.string)(['toto', 'titi', 'tata']);
        }).not.toThrowError();
        expect(function () { return decod.array(decod.string)([32, 32, 12]); }).toThrowError("at index: \"0\"\n\tScalarDecoderError: expected type: \"string\" but got type: \"number\" for value: \"32\"");
        expect(function () { return decod.array(decod.string)('Hello'); }).toThrowError("expected type: \"array\" but got type: \"string\" for value: \"Hello\"");
        expect(decod.array(decod.string)(['toto', 'titi', 'tata'])).toEqual([
            'toto',
            'titi',
            'tata',
        ]);
    });
    it('Should decode one of things', function () {
        expect(function () {
            return decod.oneOf(decod.string, decod.boolean)(true);
        }).not.toThrowError();
        expect(function () { return decod.oneOf(decod.string, decod.boolean)(32); }).toThrowError("all decoders failed\n\tScalarDecoderError: expected type: \"string\" but got type: \"number\" for value: \"32\"\n\tScalarDecoderError: expected type: \"boolean\" but got type: \"number\" for value: \"32\"");
        expect(decod.oneOf(decod.string, decod.boolean)(true)).toEqual(true);
    });
});
//# sourceMappingURL=index.test.js.map