(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports", "./errors"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    const errors_1 = require("./errors");
    function oneOf(...decoders) {
        return (input) => {
            const errors = [];
            for (const decoder of decoders) {
                try {
                    return decoder(input);
                }
                catch (e) {
                    errors.push(e);
                }
            }
            throw new errors_1.OneOfDecoderError(errors);
        };
    }
    exports.oneOf = oneOf;
    /**
     * Decode an `unknown` value as a `string`.
     * Otherwise, throws a {@link ScalarDecoderError}.
     *
     * @category Primitive decoders
     */
    exports.string = (input) => {
        if (typeof input === 'string') {
            return input;
        }
        throw new errors_1.ScalarDecoderError('string', input);
    };
    /**
     * Decode an `unknown` value if it matches the expected value.
     * Otherwise, throws a {@link ScalarDecoderError}.
     *
     * It is mostly intended to be used in conjunction with {@link oneOf} for decoding groups of specific values.
     *
     * ```ts
     * const droidDecoder = decod.oneOf(
     *   decod.is('r2d2'),
     *   decod.is('c3po')
     * );
     * ```
     *
     * @category Primitive decoders
     */
    exports.is = (expectedValue) => (input) => {
        if (input === expectedValue) {
            return expectedValue;
        }
        throw new errors_1.StrictDecoderError(expectedValue, input);
    };
    /**
     * Decode an `unknown` value as a `number`.
     * Otherwise, throws a {@link ScalarDecoderError}.
     *
     * @category Primitive decoders
     */
    exports.number = (input) => {
        if (typeof input === 'number') {
            return input;
        }
        throw new errors_1.ScalarDecoderError('number', input);
    };
    /**
     * Decode an `unknown` value as a `null` value.
     * Otherwise, throws a {@link ScalarDecoderError}.
     *
     * @category Primitive decoders
     */
    exports.null_ = (input) => {
        if (input === null) {
            return input;
        }
        throw new errors_1.ScalarDecoderError('null', input);
    };
    /**
     * Decode an `unknown` value as `undefined`.
     * Otherwise, throws a {@link ScalarDecoderError}.
     *
     * @category Primitive decoders
     */
    exports.undefined_ = (input) => {
        if (typeof input === 'undefined') {
            return input;
        }
        throw new errors_1.ScalarDecoderError('undefined', input);
    };
    /**
     * Transform a `Decoder<T>` into a `Decoder<T | null | undefined>`.
     *
     * @category Decoder combinators
     */
    exports.optional = (decoder) => oneOf(decoder, exports.null_, exports.undefined_);
    /**
     * Transform a `Decoder<T>` into a `Decoder<T | null>`.
     *
     * @category Decoder combinators
     */
    exports.nullable = (decoder) => oneOf(decoder, exports.null_);
    /**
     * Decode an `unknown` value as a `boolean`.
     * Otherwise, throws a {@link ScalarDecoderError}.
     *
     * @category Primitive decoders
     */
    exports.boolean = (input) => {
        if (typeof input === 'boolean') {
            return input;
        }
        throw new errors_1.ScalarDecoderError('boolean', input);
    };
    /**
     * Decode an `unknown` value as a `Date`.
     * Otherwise, throws a {@link ScalarDecoderError}.
     *
     * @category Primitive decoders
     */
    exports.date = (input) => {
        if (input instanceof Date) {
            return input;
        }
        throw new errors_1.ScalarDecoderError('Date', input);
    };
    /**
     * Apply the provided decoder at the specified path in a complex object.
     * Path can be either:
     * - a `string` (when querying a top level field)
     * - a `number` (when querying an array index)
     * - an `Array<string | number>` (to query an arbitrarily nested field)
     *
     * @category Decoder combinators
     */
    exports.at = (path, decoder) => (input) => {
        try {
            if (typeof path === 'string' || typeof path === 'number') {
                return decoder(input[path]);
            }
            if (path.length) {
                return exports.at(path.slice(1), decoder)(input[path[0]]);
            }
        }
        catch (e) {
            if (typeof path === 'string' || typeof path === 'number') {
                throw new errors_1.AtDecoderError(path, e);
            }
            throw new errors_1.AtDecoderError(path[0], e);
        }
        return decoder(input);
    };
    /**
     * Transform a `Decoder<T>` into a `Decoder<Array<T>>`.
     *
     * @category Decoder combinators
     */
    exports.array = (decoder) => (input) => {
        if (Array.isArray(input)) {
            return input.map((value, index) => {
                try {
                    return decoder(value);
                }
                catch (e) {
                    throw new errors_1.ArrayDecoderError(index, e);
                }
            });
        }
        throw new errors_1.ScalarDecoderError('array', input);
    };
    function try_(decoder, defaultValue) {
        return oneOf(decoder, (_) => defaultValue);
    }
    exports.try_ = try_;
    /**
     * Synonym for {@link try_}
     *
     * @category Decoder combinators
     */
    exports.attempt = try_;
    /**
     * Create a decoder for arbitrary key/value pairs from a key decoder and a value decoder.
     * Effectively transforms a `Decoder<K>` and a `Decoder<V>` into a `Decoder<Array<{ key: K, value: V }>>`.
     *
     * @category Decoder combinators
     */
    exports.assoc = (keyDecoder, valueDecoder) => (input) => Object.keys(input).map(key => ({
        key: keyDecoder(key),
        value: valueDecoder(input[key]),
    }));
    /**
     * Create a decoder for some complex structure from an object where each value is a `Decoder` and the keys
     * correspond to the desired output structure.
     *
     * ```ts
     * const personDecoder = decod.props({
     *   first_name: decod.at('first_name', decod.string),
     *   last_name: decod.at('last_name', decod.string),
     *   age: decod.at('age', decod.number),
     * });
     * ```
     *
     * @category Decoder combinators
     */
    exports.props = (obj) => (input) => Object.keys(obj).reduce((acc, key) => {
        try {
            const decoded = obj[key](input);
            return Object.assign(Object.assign({}, acc), { [key]: decoded });
        }
        catch (e) {
            throw new errors_1.PropsDecoderError(key, e);
        }
    }, {});
});
//# sourceMappingURL=decoders.js.map